@@ -70,7 +70,7 @@ without modification, or license it as you see fit.
 #define TRUE  1
 #define FALSE 0
 
-#define ID "$Id: tk_interp.cc,v 1.5 2005/10/31 03:32:51 pkienzle Exp $"
+#define ID "$Id: tk_interp.cc,v 1.6 2006/05/19 06:58:50 jwe Exp $"
 
 static Tcl_Interp *interp = NULL;
 
@@ -946,7 +946,7 @@ See also: tk_start, tk_end, tk_loop")
    int nargin = args.length();
    if (nargin == 0)
      {
-       print_usage("tk_cmd");
+       print_usage ();
        return ret;
      }
 
