@@ -1,26 +1,32 @@
-// Copyright (C) 2004,2005  Michael Creel   <michael.creel@uab.es>
+// Copyright (C) 2004,2005,2006  Michael Creel   <michael.creel@uab.es>
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
 //  (at your option) any later version.
-// 
+//
 //  This program is distributed in the hope that it will be useful,
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
-// 
+//
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
+//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-// bfgsmin
-// 
-// bfgs or limited memory bfgs minimization of function of form 
+// bfgsmin: bfgs or limited memory bfgs minimization of function of form
 //
 // [value, return_2,..., return_m] = f(arg_1, arg_2,..., arg_n)
-// 
-// By default, minimization is w.r.t. arg_1, but this can be overridden
+//
+// By default, minimization is w.r.t. arg_1, but it can be done w.r.t. any argument
+
+// the functions defined in this file are:
+// any_bad_argument: argument checks for all functions in this file
+// __numgradient: numeric gradient, used only if analytic not supplied
+// __bisectionstep: fallback stepsize algorithm
+// __newtonstep: default stepsize algorithm
+// bfgsmin: the DLD function visible to users
+
 
 #include <oct.h>
 #include <octave/parse.h>
@@ -35,7 +41,7 @@ any_bad_argument(const octave_value_list& args)
 {
 
         int i;
-	
+
         if (!args(0).is_string())
         {
         error("bfgsmin: first argument must be string holding objective function name");
@@ -48,7 +54,6 @@ any_bad_argument(const octave_value_list& args)
 		return true;
 	}
 
-  
   	// controls should be 4 or 5 element cell array of integers
 	if (args.length() > 2)
 	{
@@ -57,15 +62,15 @@ any_bad_argument(const octave_value_list& args)
 		{
 			error("bfgsmin: 3rd argument must be a cell array of 4 or 5 integers");
 			return true;
-		}	
-		
+		}
+
         // there should be 4 or 5 elements, depending on if bfgs or lbfgs
 	if (((control.length() < 4) || (control.length() > 5)))
 	{
 		error("bfgsmin: 3rd argument must be a cell array of 4 or 5 integers");
 		return true;
 	}
-		
+
 	// special treatment for 1st element - it can be Inf or an integer
 	if (!xisinf (control(0).double_value()))
 	{
@@ -74,10 +79,10 @@ any_bad_argument(const octave_value_list& args)
 		{
 			error("bfgsmin: 3rd argument must be a cell array of 4 or 5 integers");
 			return true;
-		}	
-	}	
+		}
+	}
 
-        // now the other 3 or 4 elements	
+        // now the other 3 or 4 elements
 		for (i=1; i < control.length(); i++)
 		{
 			int tmp = control(i).int_value();
@@ -88,13 +93,13 @@ any_bad_argument(const octave_value_list& args)
 			}
 			if (i == 3) // minarg must point to one of args(1)
 			{
-				if ((tmp > args(1).length())||(tmp < 1))  
+				if ((tmp > args(1).length())||(tmp < 1))
 				{
 					error("bfgsmin: 4th element of controls must be a positive integer that indicates \n\
 which of the elements of the second argument is the one minimization is over");
 					return true;
 				}
-			}		
+			}
 			if (i == 4) // memory must be zero (bfgs) or positive integer (lbfgs)
 			{
 				if (tmp < 0)
@@ -103,12 +108,12 @@ which of the elements of the second argument is the one minimization is over");
 must be 0 (bfgs) or positive integer (memory for lbfgs)");
 					return true;
 				}
-			}		
+			}
 
 		}
 	}
 
-  
+
   	// tolerances should be 3 element cell array of doubles
   	if (args.length() > 3)
   	{
@@ -118,7 +123,7 @@ must be 0 (bfgs) or positive integer (memory for lbfgs)");
 		{
 			error("bfgsmin: 4th argument must be a cell array of 3 positive doubles");
 			return true;
-		}	
+		}
 
 		// there should be 3 elements
 		if (!(tols.length()==3))
@@ -126,7 +131,7 @@ must be 0 (bfgs) or positive integer (memory for lbfgs)");
 			error("bfgsmin: 4th argument, if supplied, must a 3-element cell array of tolerances");
 			return true;
 		}
-		
+
 		for (i=0; i<3; i++)
 		{
 			double tmp = tols(i).double_value();
@@ -134,18 +139,73 @@ must be 0 (bfgs) or positive integer (memory for lbfgs)");
 			{
 				error("bfgsmin: 4th argument must be a cell array of 3 positive doubles");
 				return true;
-			}	
+			}
 
 			if(tmp < 0)
-			{	
+			{
 				error("bfgsmin: 4th argument must be a cell array of 3 positive doubles");
 				return true;
-			}	
+			}
 		}
-	}	
+	}
   	return false;
 }
 
+
+// __numgradient: numeric central difference gradient for bfgs.
+// This is the same as numgradient, except the derivative is known to be a vector, it's defined as a column,
+// and the finite difference delta is incorporated directly rather than called from a function
+ColumnVector __numgradient(std::string f, Cell& f_args, int minarg)
+{
+	octave_value_list c_args(2,1); // for cellevall {f, f_args}
+	c_args(0) = f;
+	c_args(1) = f_args;
+	octave_value_list fdiff_args(2,1);
+	octave_value_list f_return;
+	double obj_value, SQRT_EPS, diff, delta, obj_left, obj_right, p, d, delta_right, delta_left;
+	int i, j, test;
+
+	ColumnVector parameter = f_args(minarg - 1).column_vector_value();
+
+	// initial function value
+	f_return = feval("celleval", c_args);
+	obj_value = f_return(0).double_value();
+
+	const int k = parameter.rows();
+	ColumnVector derivative(k, 1);
+
+	for (j=0; j<k; j++) // get 1st derivative by central difference
+	{
+		p = parameter(j);
+		// determine delta for finite differencing
+		SQRT_EPS = sqrt(DBL_EPSILON);
+		diff = exp(log(DBL_EPSILON)/3);
+		test = (fabs(p) + SQRT_EPS) * SQRT_EPS > diff;
+		if (test) delta = (fabs(p) + SQRT_EPS) * SQRT_EPS;
+		else delta = diff;
+		// right side
+		parameter(j) = d = p + delta;
+		delta_right = d - p;
+		f_args(minarg - 1) = parameter;
+		c_args(1) = f_args;
+		f_return = feval("celleval", c_args);
+		obj_right = f_return(0).double_value();
+		// left size
+		d = p - delta;
+		parameter(j) = d;
+		delta_left = p - d;
+		f_args(minarg - 1) = parameter;
+		c_args(1) = f_args;
+		f_return = feval("celleval", c_args);
+		obj_left = f_return(0).double_value();
+
+		parameter(j) = p;  // restore original parameter
+		derivative(j) = (obj_right - obj_left) / (delta_right + delta_left);
+	}
+	return derivative;
+}
+
+
 // this is the lbfgs direction, used if control has 5 elements
 ColumnVector lbfgs_recursion(int memory, Matrix& sigmas, Matrix& gammas, ColumnVector d)
 {
@@ -162,7 +222,7 @@ ColumnVector lbfgs_recursion(int memory, Matrix& sigmas, Matrix& gammas, ColumnV
 	  		d = cond*d;
 		}
    		 return d;
-  	}	
+  	}
   	else
   	{
     		const int k = d.rows();
@@ -179,7 +239,146 @@ ColumnVector lbfgs_recursion(int memory, Matrix& sigmas, Matrix& gammas, ColumnV
     		d = d + (alpha - rho * gam.transpose() * d) * sig;
   	}
   	return d;
-}			
+}
+
+// __bisectionstep: fallback stepsize method if __newtonstep fails
+octave_value_list __bisectionstep(std::string f, Cell& f_args, ColumnVector dx, int minarg)
+{
+	double obj_0, obj, a;
+	octave_value_list f_return;
+	octave_value_list c_args(2,1); // for cellevall {f, f_args}
+	octave_value_list stepobj(2,1);
+	int found_improvement;
+
+	ColumnVector x (f_args(minarg - 1).column_vector_value());
+	ColumnVector x_in = x;
+
+	// possibly function returns a cell array
+	// obj. value will be in first position
+	c_args(0) = f;
+	c_args(1) = f_args;
+	f_return = feval("celleval", c_args);
+	obj_0 = f_return(0).double_value();
+	a = 1.0;
+	found_improvement = 0;
+	// this first loop goes until an improvement is found
+	while (a > 2*DBL_EPSILON) // limit iterations
+	{
+		f_args(minarg - 1) = x + a*dx;
+		c_args(1) = f_args;
+		f_return = feval("celleval", c_args);
+		obj = f_return(0).double_value();
+		// reduce stepsize if worse, or if function can't be evaluated
+		if ((obj >= obj_0) || lo_ieee_isnan(obj)) a = 0.5 * a;
+		else
+		{
+			obj_0 = obj;
+			found_improvement = 1;
+			break;
+		}
+	}
+	// If unable to find any improvement break out with stepsize zero
+	if (!found_improvement)
+	{
+		stepobj(0) = 0.0;
+		stepobj(1) = obj_0;
+		return stepobj;
+	}
+	// now keep going until we no longer improve, or reach max trials
+	while (a > 2*DBL_EPSILON)
+	{
+		a = 0.5*a;
+		f_args(minarg - 1) = x + a*dx;
+		c_args(1) = f_args;
+		f_return = feval("celleval", c_args);
+		obj = f_return(0).double_value();
+		// if improved, record new best and try another step
+		if ((obj < obj_0) & !lo_ieee_isnan(obj)) obj_0 = obj;
+		else
+		{
+			a = a / 0.5; // put it back to best found
+			break;
+		}
+	}
+	stepobj(0) = a;
+	stepobj(1) = obj_0;
+	return stepobj;
+}
+
+// __newtonstep: default stepsize algorithm
+octave_value_list __newtonstep(std::string f, Cell& f_args, ColumnVector dx, int minarg)
+{
+	double obj, obj_0, obj_left, obj_right, delta, a, gradient, hessian;
+	octave_value_list f_return;
+	octave_value_list c_args(2,1); // for cellevall {f, f_args}
+	octave_value_list stepobj(2,1);
+
+	ColumnVector x (f_args(minarg - 1).column_vector_value());
+	ColumnVector x_in = x;
+	gradient = 1.0;
+
+	// possibly function return cell array
+	// obj. value will be in first position
+	c_args(0) = f;
+	c_args(1) = f_args;
+	f_return = feval("celleval", c_args);
+	obj = f_return(0).double_value();
+	obj_0 = obj;
+	delta = 0.001; // experimentation show that this is a good choice
+	ColumnVector x_right = x + delta*dx;
+	ColumnVector x_left = x  - delta*dx;
+
+	// possibly function return cell array
+	// obj. value will be in first position
+	f_args(minarg - 1) = x_right;
+	c_args(1) = f_args;
+	f_return = feval("celleval", c_args);
+	obj_right = f_return(0).double_value();
+
+	f_args(minarg - 1) = x_left;
+	c_args(1) = f_args;
+	f_return = feval("celleval", c_args);
+	obj_left = f_return(0).double_value();
+
+	gradient = (obj_right - obj_left) / (2*delta);  // take central difference
+	hessian =  (obj_right - 2*obj + obj_left) / pow(delta, 2.0);
+	hessian = fabs(hessian); // ensures we're going in a decreasing direction
+	if (hessian <= 2*DBL_EPSILON) hessian = 1.0; // avoid div by zero
+	a = - gradient / hessian;  // hessian inverse gradient: the Newton step
+	if (a < 0) 	// since direction is descending, a must be positive
+	{ 		// if it is not, go to bisection step
+		f_args(minarg - 1) = x_in;
+		f_return = __bisectionstep(f, f_args, dx, minarg);
+		a = f_return(0).double_value();
+		obj = f_return(1).double_value();
+		stepobj(0) = a;
+		stepobj(1) = obj;
+		return octave_value_list(stepobj);
+	}
+
+	a = (a < 5.0)*a + 5.0*(a>=5.0); // Let's avoid extreme steps that might cause crashes
+
+	// ensure that this is improvement
+	f_args(minarg - 1) = x + a*dx;
+	c_args(1) = f_args;
+	f_return = feval("celleval", c_args);
+	obj = f_return(0).double_value();
+
+	// if not, fall back to bisection
+	if ((obj > obj_0) || lo_ieee_isnan(obj))
+	{
+		f_args(minarg - 1) = x_in;
+		f_return = __bisectionstep(f, f_args, dx, minarg);
+		a = f_return(0).double_value();
+		obj = f_return(1).double_value();
+	}
+
+	stepobj(0) = a;
+	stepobj(1) = obj;
+	return stepobj;
+}
+
+
 
 
 DEFUN_DLD(bfgsmin, args, ,
@@ -219,28 +418,7 @@ Returns:\n\
 * convergence: 1 if normal conv, other values if not\n\
 * iters: number of iterations performed\n\
 \n\
-Example:\n\
-function a = f(x,y)\n\
-	a = x'*x + y'*y;\n\
-endfunction\n\
-\n\
-In this example, x is optimized since it's the first\n\
-element of the cell array, y is a fixed constant = 1\n\
-\n\
-bfgsmin(\"f\", {ones(2,1), 1}, {10,2,1,1})\n\
-\n\
-bfgsmin final results: Iteration 1\n\
-Stepsize 0.0000000\n\
-Objective function value     1.0000000000\n\
-Function conv 1  Param conv 1  Gradient conv 1\n\
-  params  gradient  change\n\
-  0.0000   0.0000   0.0000\n\
-  0.0000   0.0000   0.0000\n\
-\n\
-ans =\n\
-\n\
-   6.1497e-12\n\
-   6.1497e-12\n\
+Example: see bfgsmin_example\n\
 ")
 {
 	int nargin = args.length();
@@ -255,29 +433,22 @@ ans =\n\
 
 	std::string f (args(0).string_value());
   	Cell f_args (args(1).cell_value());
-
-  	octave_value_list step_args(4,1);  // for stepsize: {f, f_args, direction, minarg}
-  	octave_value_list g_args(3,1); // for numgradient {f, f_args, minarg}
-  	octave_value_list c_args(2,1); // for celleval {f, f_args}  
-	
+	octave_value_list c_args(2,1); // for celleval {f, f_args}
   	octave_value_list f_return; // holder for feval returns
 
-	int max_iters, verbosity, criterion, minarg;
-	int convergence, iter;
-	int memory, gradient_failed, i, j, k, conv_fun, conv_param, conv_grad;
-	int have_gradient = 0;
-	double func_tol, param_tol, gradient_tol, stepsize, obj_value;
-	double obj_in, last_obj_value, denominator, test;
-	Matrix H, tempmatrix, H1, H2;
+	int max_iters, verbosity, criterion, minarg, convergence, iter, memory, \
+		gradient_failed, i, j, k, conv_fun, conv_param, conv_grad, have_gradient;
+	double func_tol, param_tol, gradient_tol, stepsize, obj_value, obj_in, last_obj_value, denominator, test;
+	Matrix H, H1, H2;
 	ColumnVector thetain, d, g, g_new, p, q, sig, gam;
 
-
 	// Default values for controls
 	max_iters = INT_MAX; // no limit on iterations
-	verbosity = 0; // by default don't report results to screen
+	verbosity = 0; // don't report results to screen
 	criterion = 1; // strong convergence required
-	minarg = 1; // by default, first arg is one over which we minimize
-	      memory = 0;
+	minarg = 1; // first arg is one over which we minimize
+	memory = 0; // use regular bfgs
+	have_gradient = 0; // numeric gradient
 
 	// use provided controls, if applicable
 	if (args.length() > 2)
@@ -291,11 +462,10 @@ ans =\n\
 		criterion = control(2).int_value();
 		minarg = control(3).int_value();
 		if (control.length() > 4) memory = control(4).int_value();
-	}     
- 
+	}
 
 	// type checking for minimization parameter done here, since we don't know minarg
-	// until now	
+	// until now
 	if (!(f_args(minarg - 1).is_real_matrix() || (f_args(minarg - 1).is_real_scalar())))
 	{
 		error("bfgsmin: minimization must be with respect to a column vector");
@@ -305,9 +475,8 @@ ans =\n\
 	{
 		error("bfgsmin: minimization must be with respect to a column vector");
 		return octave_value_list();
-	}	
+	}
 
-	
 	// use provided tolerances, if applicable
 	if (args.length() == 4)
 	{
@@ -315,39 +484,27 @@ ans =\n\
 		func_tol = tols(0).double_value();
 		param_tol = tols(1).double_value();
 		gradient_tol = tols(2).double_value();
-	}     
+	}
 	else
 	{
 		// Default values for tolerances
 		func_tol  = 1e-12;
 		param_tol = 1e-6;
-		gradient_tol = 1e-4;  
+		gradient_tol = 1e-4;
 	}
 
-
-	// arguments for stepsize function
-	step_args(0) = f; 
-	step_args(1) = f_args;
-	step_args(3) = minarg;
-
-	// arguments for numgradient
-	g_args(0) = f;
-	g_args(1) = f_args;
-	g_args(2) = minarg;
-
 	// arguments for celleval (to calculate objective function value)
 	c_args(0) = f;
 	c_args(1) = f_args;
-	
+
 	// get the minimization argument
 	ColumnVector theta  = f_args(minarg - 1).column_vector_value();
 	k = theta.rows();
-	
+
 	// containers for items in limited memory version
 	Matrix sigmas(k,memory);
 	Matrix gammas(k,memory);
 
-	
 	// initialize things
 	convergence = -1; // if this doesn't change, it means that maxiters were exceeded
 	thetain = theta;
@@ -356,13 +513,13 @@ ans =\n\
 	// Initial obj_value
 	f_return = feval("celleval", c_args);
 	obj_in = f_return(0).double_value();
-	if (error_state) // check that objective function returns a double
-	{
+
+	// check that objective function returns a double
+	if (error_state)
+		{
 		error("bfgsmin: objective function did not return a scalar numeric value");
         	return octave_value_list();
-	}      
-
-	last_obj_value = obj_in;
+	}
 
 	// maybe we have analytic gradient?
 	// if the function returns more than one item, and the second
@@ -374,21 +531,19 @@ ans =\n\
         		if ((f_return(1).rows() == k) & (f_return(1).columns() == 1))
 			{
 				g = f_return(1).column_vector_value();
-				have_gradient = 1; // future reference
+				have_gradient = 1; // save for future reference
 			}
 		}
 		else have_gradient = 0;
 	}
 	if (!have_gradient) // use numeric gradient
 	{
-		f_return = feval("numgradient", g_args);
+		g = __numgradient(f, f_args, minarg);
 		if (error_state)
 		{
 			error("bfgsmin: unable to calculate numeric gradient of objective function: need to recompile numgradient?");
 			return octave_value_list();
 		}
-		tempmatrix = f_return(0).matrix_value();
-		g = tempmatrix.row((octave_idx_type)0).transpose();
 	}
 
 	// check that gradient is ok
@@ -403,12 +558,13 @@ ans =\n\
 		convergence = 2;
 	}
 
+	last_obj_value = obj_in; // initialize, is updated after each iteration
 	// MAIN LOOP STARTS HERE
 	for (iter = 0; iter < max_iters; iter++)
 	{
   		// make sure the messages aren't stale
-		conv_fun = -1; 
-		conv_param =  -1;
+		conv_fun = -1;
+		conv_param = -1;
 		conv_grad = -1;
 
     		if(memory > 0) // lbfgs
@@ -420,23 +576,18 @@ ans =\n\
 		else d = -H*g; // ordinary bfgs
 
 		// stepsize: try (l)bfgs direction, then steepest descent if it fails
-		step_args(2) = d;
 		f_args(minarg - 1) = theta;
-		step_args(1) = f_args;
-		f_return = feval("newtonstep", step_args);
+		f_return = __newtonstep(f, f_args, d, minarg);
 		stepsize = f_return(0).double_value();
 		obj_value = f_return(1).double_value();
 		if (stepsize == 0.0) // fall back to steepest descent
 		{
 			d = -g; // try steepest descent
-			step_args(2) = d;
-			f_return = feval("newtonstep", step_args);
+			f_return = __newtonstep(f, f_args, d, minarg);
 			stepsize = f_return(0).double_value();
 			obj_value = f_return(1).double_value();
 		}
-		
 		p = stepsize*d;
-
 		// check normal convergence: all 3 must be satisfied
 		// function convergence
 		if (fabs(last_obj_value) > 1.0)
@@ -450,10 +601,11 @@ ans =\n\
 		// parameter change convergence
 		test = sqrt(theta.transpose() * theta);
 		if (test > 1) conv_param = sqrt(p.transpose() * p) / test < param_tol ;
-		else conv_param = sqrt(p.transpose() * p) < param_tol;
+		else conv_param = sqrt(p.transpose() * p) < param_tol;		// Want intermediate results?
+		if ((verbosity > 0) && (verbosity < 2))
+
 		// gradient convergence
 		conv_grad = sqrt(g.transpose() * g) < gradient_tol;
-
 		// Want intermediate results?
 		if ((verbosity > 0) && (verbosity < 2))
 		{
@@ -465,7 +617,7 @@ ans =\n\
 			else printf("Using numeric gradient\n");
 			printf("\n");
 			printf("------------------------------------------------------\n");
-			printf("Function conv %d  Param conv %d  Gradient conv %d\n", conv_fun, conv_param, conv_grad); 
+			printf("Function conv %d  Param conv %d  Gradient conv %d\n", conv_fun, conv_param, conv_grad);
 			printf("------------------------------------------------------\n");
 			printf("Objective function value %g\n", last_obj_value);
 			printf("Stepsize %g\n", stepsize);
@@ -474,7 +626,6 @@ ans =\n\
 			printf("\n param	gradient  change\n");
 			for (j = 0; j<k; j++) printf("%8.4f %8.4f %8.4f\n",theta(j),g(j),p(j));
 		}
-		
 		// Are we done?
 		if (criterion == 1)
 		{
@@ -489,11 +640,8 @@ ans =\n\
 			convergence = 1;
 			break;
 		}
-    	    
-    		    
-		last_obj_value = obj_value; 
+		last_obj_value = obj_value;
 		theta = theta + p;
-
 		// new gradient
 		f_args(minarg - 1) = theta;
 		if (have_gradient)
@@ -504,17 +652,13 @@ ans =\n\
 		}
 		else // use numeric gradient
 		{
-			g_args(1) = f_args;
-			f_return = feval("numgradient", g_args);
-			tempmatrix = f_return(0).matrix_value();
-			g_new = tempmatrix.row((octave_idx_type)0).transpose();
+			g_new = __numgradient(f, f_args, minarg);
 		}
-		
+
 		// Check that gradient is ok
 		gradient_failed = 0;  // test = 1 means gradient failed
 		for (i=0; i<k;i++) gradient_failed = gradient_failed + xisnan(g_new(i));
-
-		if (memory == 0) //use bfgs
+		if (memory == 0) //bfgs?
 		{
 			// Hessian update if gradient ok
 			if (!gradient_failed)
@@ -539,7 +683,7 @@ ans =\n\
 			else H = identity_matrix(k,k); // reset hessian if gradient fails
 			// then try to start again with steepest descent
 		}
-		else // use lbfgs
+		else // otherwise lbfgs
 		{
 			// save components for Hessian if gradient ok
 			if (!gradient_failed)
@@ -569,9 +713,9 @@ ans =\n\
 				gammas.fill(0.0);
 				theta = theta - p;
 			}
-		}		
+		}
 	}
-	
+
 	// Want last iteration results?
 	if (verbosity > 0)
 	{
@@ -587,7 +731,7 @@ ans =\n\
 		if ((convergence == 1) & (criterion == 1))  printf("STRONG CONVERGENCE\n");
 		if ((convergence == 1) & !(criterion == 1)) printf("WEAK CONVERGENCE\n");
 		if (convergence == 2)                       printf("NO CONVERGENCE: algorithm failed\n");
-		printf("Function conv %d  Param conv %d  Gradient conv %d\n", conv_fun, conv_param, conv_grad);	
+		printf("Function conv %d  Param conv %d  Gradient conv %d\n", conv_fun, conv_param, conv_grad);
 		printf("------------------------------------------------------\n");
 		printf("Objective function value %g\n", last_obj_value);
 		printf("Stepsize %g\n", stepsize);
